@@ -87,7 +87,7 @@ def build_show_options(options=None):
         open_browser=False,
         notebook=False,
         force=False,
-        ignore_duplicate=False,
+        ignore_duplicate=True,
         app_root=None,
         allow_cell_edits=True,
         inplace=False,
